﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Windows.DataPresenter;
using System.Windows.Media.Effects;
using System.ComponentModel;

namespace DragDropXamGrids
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        DataRecordPresenter moving;
        Rectangle rect;
        private bool isDragging = false;
        private XamDataGrid sourceGrid;
        private bool isFromSameGrid = false;
        public Window1()
        {
            InitializeComponent();

            xamDataGrid1.DataSource = EmployeeData.GenerateEmployeeData();
            xamDataGrid2.DataSource = new BindingList<Employee>();
            //xamDataGrid1.FieldLayouts[0].Fields[1].Settings.CellContentAlignment = CellContentAlignment.LabelAboveValueAlignCenter;
            //Wire all XamDataGrids with Drag & Drop functionality
            #region Wiring Drag&Drop
            foreach (var child in XamDataGridsLayout.Children)
            {
                if (child is XamDataGrid)
                {
                    (child as XamDataGrid).MouseRightButtonDown += xamDataGrid_MouseRightButtonDown;
                    (child as XamDataGrid).MouseRightButtonUp += xamDataGrid_MouseRightButtonUp;
                    (child as XamDataGrid).MouseMove += xamDataGrid_MouseMove;
                }
            }
            #endregion

        }
        /// <summary>
        /// Drag And Drop from and to the same grid
        /// </summary>
        /// <param name="grid">Source XamDataGrid</param>
        /// <param name="source"></param>
        /// <param name="moved">The DataRecordPresenter that is dragged over</param>
        private void DragDropSameGrid(XamDataGrid grid,DependencyObject source,DataRecordPresenter moved ) 
        {
            #region XDG-X - XDG-X
                        BindingList<Employee> b = grid.DataSource as BindingList<Employee>;
                        DragSurface.Children.Remove(rect);

                        if (source == null)
                        {
                            isDragging = false;
                            return;
                        }
                        if (moved == null || moving == null)
                        {
                            isDragging = false;
                            return;
                        }

                        if (moved != moving)
                        {


                            if ((moving.Record as DataRecord).Index > (moved.Record as DataRecord).Index)
                            {
                                b.Insert((moved.Record as DataRecord).Index, (moving.Record as DataRecord).DataItem as Employee);
                                b.RemoveAt((moving.Record as DataRecord).DataItemIndex);
                            }
                            else if ((moving.Record as DataRecord).Index != -1)
                            {
                                b.Insert((moved.Record as DataRecord).Index + 1, (moving.Record as DataRecord).DataItem as Employee);
                                b.RemoveAt((moving.Record as DataRecord).DataItemIndex);
                            }
                            grid.DataSource = b;
                            xamDataGrid1.UpdateLayout();
                        }
                        this.UpdateLayout();
                        #endregion
                    
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="grid1">Sender XamDataGrid</param>
        /// <param name="grid2">Receiver XamDataGrid</param>
        /// <param name="source"></param>
        private void DragDifferentGrids(XamDataGrid grid1, XamDataGrid grid2,DependencyObject source) 
        {
            DragSurface.Children.Remove(rect);

            BindingList<Employee> b1 = grid1.DataSource as BindingList<Employee>;
            BindingList<Employee> b2 = grid2.DataSource as BindingList<Employee>;

            if (b2 == null)
            {
                b2 = new BindingList<Employee>();
                b2.Add(moving.DataRecord.DataItem as Employee);
                b1.RemoveAt((moving.Record as DataRecord).DataItemIndex);
                grid2.DataSource = b2;
            }
            else
            {

                DataRecordPresenter movedXDG2 = Infragistics.Windows.Utilities.GetAncestorFromType(source, typeof(DataRecordPresenter), true) as DataRecordPresenter;

                if (source == null || moving == null) return;

                if (movedXDG2 == null)
                {
                    b2.Insert(b2.Count, moving.DataRecord.DataItem as Employee);
                    b1.RemoveAt((moving.Record as DataRecord).DataItemIndex);
                }
                else
                {
                    b2.Insert(movedXDG2.DataRecord.DataItemIndex, (moving.DataRecord.DataItem) as Employee);
                    b1.RemoveAt((moving.Record as DataRecord).DataItemIndex);

                }
            }
        }

        private void xamDataGrid_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            #region
            if (DragSurface.Children.Contains(rect))
            {
                DragSurface.Children.Remove(rect);
            }
            DependencyObject source = e.OriginalSource as DependencyObject;

            if (source == null)
            {
                isDragging = false;
                return;
            }

            moving = Infragistics.Windows.Utilities.GetAncestorFromType(source, typeof(DataRecordPresenter), true) as DataRecordPresenter;

            if (moving == null)
            {
                isDragging = false;
                return;
            }

            rect = new Rectangle();

            if (!DragSurface.Children.Contains(rect) && rect != null)
            {
                Canvas.SetTop(rect, e.GetPosition(null).Y + 5);
                Canvas.SetLeft(rect, e.GetPosition(null).X + 5);

                BuildDraggerFromRecord(moving);

                DragSurface.Children.Add(rect);
                isDragging = true;
                sourceGrid = (sender as XamDataGrid);
            }
            this.UpdateLayout();
            #endregion
        }

        private void xamDataGrid_MouseRightButtonUp(object sender, MouseButtonEventArgs e)
        {
            BindingList<Employee> b1 = xamDataGrid1.DataSource as BindingList<Employee>;
            BindingList<Employee> b2 = xamDataGrid2.DataSource as BindingList<Employee>;
            DependencyObject source = e.OriginalSource as DependencyObject;
            string sendersName = (sender as XamDataGrid).Name;
            if ((sender as XamDataGrid).Name == sourceGrid.Name) isFromSameGrid = true;
            DataRecordPresenter moved = Infragistics.Windows.Utilities.GetAncestorFromType(source, typeof(DataRecordPresenter), true) as DataRecordPresenter;
            try
            {
                if (isDragging)
                {

                    if (moving.DataRecord.DataItemIndex < 0) return;

                    if (isFromSameGrid)
                    {
                        DragDropSameGrid(sender as XamDataGrid, source, moved);
                        isFromSameGrid = false;
                    }
                    else
                    {
                        #region XDG1 - XDG2

                        DragDifferentGrids(sourceGrid, (sender as XamDataGrid), source);
                    }
                        #endregion
                }
            }
            catch { }
            finally
            {
                isDragging = false;
                isFromSameGrid = false;
                moving = null;
                moved = null;
            }
        }

        private void xamDataGrid_MouseMove(object sender, MouseEventArgs e)
        {
            this.UpdateLayout();
            if (rect != null)
            {
                Canvas.SetTop(rect, e.GetPosition(null).Y + 5);
                Canvas.SetLeft(rect, e.GetPosition(null).X + 5);
            }
        }
        /// <summary>
        /// Building the Drag Animation
        /// </summary>
        /// <param name="drp">The DataRecordPresenter that is being dragged</param>
        private void BuildDraggerFromRecord(DataRecordPresenter drp)
        {
            #region DragPanel

            rect.Opacity = 0.6;
            DropShadowEffect dropShadowEffect = new DropShadowEffect();
            dropShadowEffect.Color = Colors.Black;
            dropShadowEffect.BlurRadius = 15;
            dropShadowEffect.Opacity = 0.6;
            rect.Effect = dropShadowEffect;
            VisualBrush vb = new VisualBrush(drp);
            rect.Fill = vb;
            rect.Width = drp.ActualWidth;
            rect.Height = drp.ActualHeight;

            #endregion
        }  
    }
}
